import { AuthorizationManagementModels } from "@azure/arm-authorization";
import { KeyVaultManagementModels } from "@azure/arm-keyvault-profile-2020-09-01-hybrid";
import { ResourceManagementModels } from "@azure/arm-resources";
import { StorageManagementModels } from "@azure/arm-storage";
import { KeyVaultKey } from "@azure/keyvault-keys";
import * as common from "@data-heaving/common";
import * as setup from "@data-heaving/pulumi-azure-pipeline-setup";
import * as types from "./types";
export interface VirtualBootstrapEvents {
  afterResolvingConfigReaderPrincipal: setup.EnvSpecificPipelineConfigReader;
  afterADApplicationExists: types.UpsertResult<{
    application: types.Application;
  }>;
  afterADServicePrincipalExists: types.UpsertResult<{
    servicePrincipal: types.ServicePrincipal;
  }>;
  beforeApplicationCredentialsExists: types.UpsertResult<{
    application: types.Application;
    credential: CredentialInfo;
    waitTimeInSecondsIfCreated: number;
  }>;
  beforeApplicationHasEnoughPermissions: types.UpsertResult<{
    application: types.Application;
    permissionsToAssign: ReadonlyArray<types.ApplicationRequiredResourceAccess>;
    waitTimeInSecondsIfCreated: number;
  }>;
  beforeAdminConsentGranted: types.UpsertResult<{
    servicePrincipal: types.ServicePrincipal;
    permissionsToGrant: ReadonlyArray<types.ApplicationRequiredResourceAccess>;
  }>;
  bootstrapperRoleAssignmentCreatedOrUpdated: AuthorizationManagementModels.RoleAssignment;
  resourceGroupCreatedOrUpdated: ResourceManagementModels.ResourceGroupsCreateOrUpdateResponse;
  storageAccountCreatedOrUpdated:
    | StorageManagementModels.StorageAccountsUpdateResponse
    | StorageManagementModels.StorageAccountsCreateResponse;
  storageAccountBlobServicesConfigured: StorageManagementModels.BlobServicesSetServicePropertiesResponse;
  storageAccountContainerCreatedOrUpdated: StorageManagementModels.BlobContainersCreateResponse;
  keyVaultCreatedOrUpdated: KeyVaultManagementModels.VaultsCreateOrUpdateResponse;
  keyVaultAdminRoleAssignmentCreatedOrUpdated: AuthorizationManagementModels.RoleAssignment;
  keyVaultAuthenticationSecretRoleAssignmentCreatedOrUpdated: AuthorizationManagementModels.RoleAssignment;
  keyCreatedOrUpdated: types.UpsertResult<{
    key: KeyVaultKey;
  }>;
  authenticationSecretCreatedOrUpdated: types.UpsertResult<{
    secretName: string;
    secretValue: string;
  }>;
}
export declare type CredentialInfo = Pick<
  types.ApplicationCredential,
  "key" | "keyId" | "type" | "usage" | "customKeyIdentifier"
>;
export declare type BootstrapEventEmitter =
  common.EventEmitter<VirtualBootstrapEvents>;
export declare const createBootstrapEventEmitterBuilder: () => common.EventEmitterBuilder<VirtualBootstrapEvents>;
export declare const consoleLoggingBootstrapEventEmitterBuilder: (
  logSubscriptionId?: boolean | undefined,
  logMessagePrefix?: Parameters<typeof common.createConsoleLogger>[0],
  builder?: common.EventEmitterBuilder<VirtualBootstrapEvents> | undefined,
  consoleAbstraction?: common.ConsoleAbstraction | undefined,
) => common.EventEmitterBuilder<VirtualBootstrapEvents>;
