"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCurrentPrincipalInfo = exports.ensureBootstrapSPIsConfigured = void 0;
const t = __importStar(require("io-ts"));
const validation = __importStar(require("@data-heaving/common-validation"));
const common = __importStar(require("@data-heaving/common"));
const crypto = __importStar(require("crypto"));
const uuid = __importStar(require("uuid"));
const types = __importStar(require("./types"));
const utils = __importStar(require("./run-common"));
const certs = __importStar(require("./run-certs"));
const util_1 = require("util");
const ensureBootstrapSPIsConfigured = async ({ eventEmitter, graphClient, bootstrapperApp: { displayName, willNeedToCreateAADApps }, certificatePEM, }) => {
    // Ensure that app ("Enterprise Application") exists
    const app = await ensureBootstrapperAppExists(eventEmitter, graphClient, displayName);
    const { appId } = app;
    // Concurrently,
    // - ensure that service principal("App Registration") exists, and
    // - ensure that app has correct permissions configured (if needed)
    // - ensure that app has correct certificate-based authentication configured
    const [sp] = await Promise.all([
        ensureBootstrapperSPExists(eventEmitter, graphClient, appId),
        willNeedToCreateAADApps
            ? grantAppPermissions(eventEmitter, graphClient, app)
            : undefined,
        ensureCertificateAuthenticationExists(eventEmitter, graphClient, app, certificatePEM),
    ]);
    if (willNeedToCreateAADApps) {
        await grantAdminConsent(eventEmitter, graphClient, sp);
    }
    return {
        clientId: appId,
        principalId: sp.id,
    };
};
exports.ensureBootstrapSPIsConfigured = ensureBootstrapSPIsConfigured;
const appGraphApi = "/applications";
const ensureBootstrapperAppExists = async (eventEmitter, client, bootstrapperAppName) => {
    // Don't cache result of client.api as it is stateful
    // TODO fallback to list-owned-objects ( https://docs.microsoft.com/en-us/graph/api/serviceprincipal-list-ownedobjects?view=graph-rest-1.0&tabs=http ) if we get:
    // statusCode: 403,
    // code: 'Authorization_RequestDenied',
    // body: '{"code":"Authorization_RequestDenied","message":"Insufficient privileges to complete the operation.","innerError":{"date":"2021-09-11T08:43:56","request-id":"9bf79d05-2480-4e88-afaa-68f0825c969b","client-request-id":"d42c428b-0789-f35e-040a-d8ef199f3296"}}'
    const existingApp = validation.decodeOrThrow(graphAPIListOf(types.application).decode, await client
        .api(appGraphApi)
        .filter(`displayName eq '${bootstrapperAppName}'`)
        .get()).value[0];
    const application = existingApp ??
        validation.decodeOrThrow(types.application.decode, await client.api(appGraphApi).post({
            displayName: bootstrapperAppName,
        }));
    eventEmitter.emit("afterADApplicationExists", {
        application,
        createNew: !existingApp,
    });
    return application;
};
const spGraphApi = "/servicePrincipals";
const ensureBootstrapperSPExists = async (eventEmitter, client, bootstrapperAppId) => {
    const existingSP = await getSPByAppId(client, bootstrapperAppId);
    const servicePrincipal = existingSP ??
        validation.decodeOrThrow(types.servicePrincipal.decode, await client.api(spGraphApi).post({
            appId: bootstrapperAppId,
        }));
    eventEmitter.emit("afterADServicePrincipalExists", {
        servicePrincipal,
        createNew: !existingSP,
    });
    return servicePrincipal;
};
const getSPByAppId = async (client, appId) => validation.decodeOrThrow(graphAPIListOf(types.servicePrincipal).decode, await client.api(spGraphApi).filter(`appId eq '${appId}'`).get()).value[0];
const ensureCertificateAuthenticationExists = async (eventEmitter, client, application, certificatePem) => {
    const { id, keyCredentials } = application;
    const certificatePattern = new RegExp(`${certs.BEGIN_CERTIFICATE.source}${/(\n\r?|\r\n?)([A-Za-z0-9+/\n\r]+=*)(\n\r?|\r\n?)(-+END CERTIFICATE-+)/
        .source}`);
    const firstCertBase64 = certificatePattern.exec(certificatePem)?.[3];
    if (!firstCertBase64) {
        throw new Error("Invalid certificate PEM contents, make sure BEGIN CERTIFICATE and END CERTIFICATE pre- and suffixes are present.");
    }
    const customKeyIdentifier = crypto
        .createHash("sha1")
        .update(Buffer.from(firstCertBase64, "base64"))
        .digest("hex")
        .toUpperCase();
    const existingKey = keyCredentials.find((keyCredential) => keyCredential.customKeyIdentifier === customKeyIdentifier);
    const createNew = !existingKey;
    const credential = existingKey ?? {
        type: "AsymmetricX509Cert",
        usage: "Verify",
        keyId: uuid.v4(),
        key: Buffer.from(certificatePem).toString("base64"),
        customKeyIdentifier,
    };
    const waitTimeInSecondsIfCreated = 120; // Sometimes even 60-90sec doesn't work... AAD be ridicilously slow.
    eventEmitter.emit("beforeApplicationCredentialsExists", {
        application,
        credential: Object.fromEntries(Object.entries(credential)),
        createNew,
        waitTimeInSecondsIfCreated,
    });
    if (createNew) {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const { customKeyIdentifier: _, ...credentialToPassToApi } = credential;
        // Please notice, the tricky part about /applications/${appId}/addKey, from https://docs.microsoft.com/en-us/graph/api/application-addkey?view=graph-rest-1.0&tabs=javascript
        // "Applications that don’t have any existing valid certificates (no certificates have been added yet, or all certificates have expired), won’t be able to use this service action. You can use the Update application operation to perform an update instead."
        await client.api(`${appGraphApi}/${id}`).patch({
            keyCredentials: [...keyCredentials, credentialToPassToApi],
        });
        // We must wait some time to let the AAD eventually consistent DB to catch up - if we don't do this, we will get an error when we try to sign in with SP right after this
        await common.sleep(waitTimeInSecondsIfCreated * 1000);
    }
};
const appRequiredPermissions = [
    {
        // resourceAppId: "00000003-0000-0000-c000-000000000000", // Microsoft Graph
        // resourceAccess: [
        //   {
        //     type: "Role",
        //     id: "18a4783c-866b-4cc7-a460-3d5e5662c884", // Application.ReadWrite.OwnedBy
        //   },
        // ],
        // We must grant permissions for AAD graph, since Pulumi AAD provider uses that instead of Microsoft Graph
        resourceAppId: "00000002-0000-0000-c000-000000000000",
        resourceAccess: [
            {
                type: "Role",
                id: "824c81eb-e3f8-4ee6-8f6d-de7f50d565b7", // Application.ReadWrite.OwnedBy
            },
        ],
    },
];
const grantAppPermissions = async (eventEmitter, client, application) => {
    const { id } = application;
    const patchableAccess = createPatchableRequiredAccessArray(application.requiredResourceAccess, appRequiredPermissions);
    const createNew = patchableAccess.length > 0;
    const waitTimeInSecondsIfCreated = 60;
    eventEmitter.emit("beforeApplicationHasEnoughPermissions", {
        application,
        permissionsToAssign: patchableAccess,
        createNew,
        waitTimeInSecondsIfCreated,
    });
    if (createNew) {
        await client.api(`${appGraphApi}/${id}`).patch({
            requiredResourceAccess: patchableAccess,
        });
        await common.sleep(waitTimeInSecondsIfCreated * 1000);
    }
};
const getResourceSPMap = async (client, ids) => (await Promise.all(ids.map(async ({ id, isSPId }) => isSPId
    ? validation.decodeOrThrow(types.servicePrincipal.decode, await client.api(`${spGraphApi}/${id}`).get())
    : await getSPByAppId(client, id)))).reduce((curMap, sp, idx) => {
    if (!sp) {
        throw new Error(`Could not find service principal: ${JSON.stringify(ids[idx])}.`);
    }
    curMap[sp.appId] = sp; // In case of AAD, the appID will be the "00000002-0000-0000-c000-000000000000", which also is resourceAppId of types.ApplicationRequiredResourceAccess
    return curMap;
}, {});
const grantAdminConsent = async (eventEmitter, client, servicePrincipal) => {
    // We need to find out which permission we need to grant admin consent for
    const spRoleAssignmentsApi = `${spGraphApi}/${servicePrincipal.id}/appRoleAssignments`;
    const adminConsentedRoleAssignments = validation.decodeOrThrow(graphAPIListOf(types.servicePrincipalAppRoleAssignment).decode, await client.api(spRoleAssignmentsApi).get()).value;
    // Get target SPs
    const consentedResourceSPs = await getResourceSPMap(client, common.deduplicate(adminConsentedRoleAssignments.map(({ resourceId }) => ({
        id: resourceId,
        isSPId: true,
    })), ({ id }) => id));
    const adminConsentedRoleAssignmentsMap = adminConsentedRoleAssignments.reduce((curMap, roleAssignment) => {
        common.getOrAddGeneric(curMap, roleAssignment.resourceId, () => ({}))[roleAssignment.appRoleId] = roleAssignment;
        return curMap;
    }, {});
    const adminConsentNeededOn = appRequiredPermissions
        .flatMap(({ resourceAppId, resourceAccess }) => {
        const resourceSP = consentedResourceSPs[resourceAppId];
        const missingResources = resourceSP
            ? resourceAccess.filter((access) => !adminConsentedRoleAssignmentsMap[resourceSP.id]?.[access.id])
            : resourceAccess;
        return missingResources.length > 0
            ? { resourceAppId, resourceAccess: missingResources }
            : undefined;
    })
        .filter((roleAssignment) => !!roleAssignment);
    const createNew = adminConsentNeededOn.length > 0;
    eventEmitter.emit("beforeAdminConsentGranted", {
        createNew,
        permissionsToGrant: adminConsentNeededOn,
        servicePrincipal,
    });
    if (createNew) {
        const consentResourceSPs = await getResourceSPMap(client, adminConsentNeededOn.map(({ resourceAppId }) => ({
            id: resourceAppId,
            isSPId: false,
        })));
        (await Promise.all(adminConsentNeededOn.flatMap(({ resourceAppId, resourceAccess }) => {
            return resourceAccess.map((resource) => {
                return client.api(spRoleAssignmentsApi).post({
                    principalId: servicePrincipal.id,
                    resourceId: consentResourceSPs[resourceAppId].id,
                    appRoleId: resource.id,
                });
            });
        }))).map((response) => validation.decodeOrThrow(types.servicePrincipalAppRoleAssignment.decode, response));
    }
};
const createPatchableRequiredAccessArray = (existingPermissions, additionalPermissions) => {
    const permissionAddInfos = [];
    // One can get all possible permissions via https://graph.windows.net/myorganization/applicationRefs/00000003-0000-0000-c000-000000000000?api-version=2.0&lang=en
    // And then examining items in "oauth2Permissions" array
    // Notice!!!! the Pulumi azuread provider uses azure-sdk-for-go, which underneath uses graphrbac, which underneath uses legacy Azure AD Graph endpoint ( https://graph.windows.net/ )!
    // This is why we must, instead of adding Microsoft Graph permissions, add Azure AD Graph permissions!
    for (const additionalPermission of additionalPermissions) {
        addToPatchablePermissions(existingPermissions, permissionAddInfos, additionalPermission);
    }
    const patchableAccess = [];
    if (permissionAddInfos.length > 0) {
        const deduplicatedAdditionalPermissions = common.deduplicate(permissionAddInfos, ({ indexToRemove }) => `${indexToRemove}`);
        if (deduplicatedAdditionalPermissions.length < permissionAddInfos.length) {
            throw new Error("Not implemented: complex permission delta");
        }
        patchableAccess.push(...existingPermissions);
        for (const { indexToRemove, accessToAdd, } of deduplicatedAdditionalPermissions) {
            if (indexToRemove >= 0) {
                patchableAccess[indexToRemove].resourceAccess.concat(accessToAdd.resourceAccess);
            }
            else {
                patchableAccess.push(accessToAdd);
            }
        }
    }
    return patchableAccess;
};
const addToPatchablePermissions = (existingAccess, accessToAdd, requiredAccess) => {
    const existingForResourceAppIdx = existingAccess.findIndex((r) => r.resourceAppId === requiredAccess.resourceAppId);
    const missingAccess = existingForResourceAppIdx < 0
        ? []
        : requiredAccess.resourceAccess.filter((r) => !existingAccess[existingForResourceAppIdx].resourceAccess.some((a) => (0, util_1.isDeepStrictEqual)(r, a)));
    if (existingForResourceAppIdx < 0 || missingAccess.length > 0) {
        accessToAdd.push({
            indexToRemove: existingForResourceAppIdx,
            accessToAdd: existingForResourceAppIdx < 0 ||
                missingAccess.length === requiredAccess.resourceAccess.length
                ? requiredAccess
                : {
                    resourceAppId: requiredAccess.resourceAppId,
                    resourceAccess: requiredAccess.resourceAccess.concat(missingAccess),
                },
        });
    }
};
const graphAPIListOf = (item) => t.type(
// Also typically has "@odata.context" attribute but
{
    value: utils.listOf(item),
}, "APIResult");
const graphSelf = t.type({
    // ["@odata.context"]: t.literal(
    //   "https://graph.microsoft.com/v1.0/$metadata#users/$entity"
    // ),
    // ["@odata.id"]: validation.urlWithPath,
    // businessPhones: [],
    // displayName: validation.nonEmptyString,
    // givenName: validation.nonEmptyString,
    // jobTitle: null,
    // mail: null,
    // mobilePhone: null,
    // officeLocation: null,
    // preferredLanguage: validation.nonEmptyString,
    // surname: validation.nonEmptyString,
    // userPrincipalName: validation.nonEmptyString,
    id: validation.uuid,
}, "GraphUser");
const getCurrentPrincipalInfo = async (graphClient, givenClientId) => {
    // TODO handle situations when client ID is given but it is not actually used
    if (givenClientId) {
        const id = validation.decodeOrThrow(graphAPIListOf(types.servicePrincipal).decode, await graphClient
            .api(`${spGraphApi}`)
            .filter(`appId eq '${givenClientId}'`)
            .get()).value[0]?.id;
        if (!id) {
            throw new Error(`Could not find corresponding SP for client ID ${givenClientId}.`);
        }
        return {
            principalId: id,
            principalType: "ServicePrincipal",
        };
    }
    else {
        return {
            principalId: validation.decodeOrThrow(graphSelf.decode, await graphClient.api("/me").get()).id,
            principalType: "User",
        };
    }
    // try {
    // } catch (e) {
    //   if (
    //     e instanceof graph.GraphError &&
    //     e.message ===
    //       "/me request is only valid with delegated authentication flow."
    //   ) {
    //     if (givenClientId) {
    //     } else {
    //       throw new Error("Not logged in as user nor as service principal...?");
    //     }
    //   } else {
    //     throw e;
    //   }
    // }
};
exports.getCurrentPrincipalInfo = getCurrentPrincipalInfo;
//# sourceMappingURL=run-ad.js.map