"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ensureEndsWithNewline = exports.ensurePfxExists = exports.ensureCertificateCredentialsFileExists = exports.BEGIN_CERTIFICATE = exports.ensureKeyAndCertExists = void 0;
const fs = __importStar(require("fs/promises"));
const fsc = __importStar(require("fs"));
const path = __importStar(require("path"));
const child_process_1 = require("child_process");
const util_1 = require("util");
const uuid = __importStar(require("uuid"));
const execFileAsync = (0, util_1.promisify)(child_process_1.execFile);
// This is not safe against concurrent accesses, but I think it's reasonable assumption this bootstrap program is not concurrently run with same inputs
const checkFileExistsAsync = async (path) => {
    let exists = false;
    try {
        await fs.access(path, fsc.constants.R_OK);
        exists = true;
    }
    catch {
        // Ignore
    }
    return exists;
};
const ensureKeyAndCertExists = async (keyPath, certPath, rsaBits, validityPeriodInDays, certSubject) => {
    const [keyPathExists, certPathExists] = await Promise.all([keyPath, certPath].map(checkFileExistsAsync));
    const opensslReqKeyArgs = [];
    if (!keyPathExists && !certPathExists) {
        opensslReqKeyArgs.push("-newkey", `rsa:${rsaBits}`, "-keyout", keyPath);
    }
    else if (keyPathExists !== certPathExists) {
        if (keyPathExists) {
            opensslReqKeyArgs.push("-key", keyPath);
        }
        else {
            throw new Error("Cert exists but key does not - can not proceed.");
        }
    }
    if (opensslReqKeyArgs.length > 0) {
        await execFileAsync("openssl", [
            "req",
            "-x509",
            "-out",
            certPath,
            "-nodes",
            "-days",
            `${validityPeriodInDays}`,
            "-subj",
            certSubject,
            ...opensslReqKeyArgs,
        ], {
            env: {},
            shell: false,
        });
    }
};
exports.ensureKeyAndCertExists = ensureKeyAndCertExists;
exports.BEGIN_CERTIFICATE = /(-+BEGIN CERTIFICATE-+)/;
const ensureCertificateCredentialsFileExists = async (tempDir, keyPath, certificatePEM) => {
    const keyAndCertPath = path.join(tempDir, `auth-${uuid.v4()}.pem`);
    const keyPEM = (0, exports.ensureEndsWithNewline)(await fs.readFile(keyPath, "utf-8"));
    await fs.writeFile(keyAndCertPath, `${keyPEM}${certificatePEM}`);
    return {
        keyPEM,
        keyAndCertPath,
    };
};
exports.ensureCertificateCredentialsFileExists = ensureCertificateCredentialsFileExists;
const ensurePfxExists = async (keyPath, certPath, pfxPath, pfxPassword) => {
    const passwordEnvName = "PFX_PASSWORD";
    if (!(await checkFileExistsAsync(pfxPath))) {
        await execFileAsync("openssl", [
            "pkcs12",
            "-export",
            "-inkey",
            keyPath,
            "-in",
            certPath,
            "-out",
            pfxPath,
            "-password",
            `env:${passwordEnvName}`,
        ], {
            env: {
                [passwordEnvName]: pfxPassword,
            },
            shell: false,
        });
    }
    return pfxPath;
};
exports.ensurePfxExists = ensurePfxExists;
const ensureEndsWithNewline = (str) => str.endsWith("\n") ? str : `${str}\n`;
exports.ensureEndsWithNewline = ensureEndsWithNewline;
//# sourceMappingURL=run-certs.js.map