import "cross-fetch";
import "cross-fetch/polyfill";
import * as id from "@azure/identity";
import * as pulumi from "@data-heaving/pulumi-azure-pipeline-setup";
import * as pulumiAzure from "@data-heaving/pulumi-azure";
import * as types from "./types";
import * as events from "./events";
import * as pulumiSetup from "./run-pulumi";
export interface Inputs {
  eventEmitter: events.BootstrapEventEmitter;
  credentials: types.BootstrappingCredentials;
  bootstrapperApp: BootstrapperApp;
  azure: pulumiAzure.AzureCloudInformationFull;
  organization: pulumiSetup.OrganizationInfo;
  pulumiEncryptionKeyBits: number;
}
export declare type BootstrapperApp = BootstrapperAppSP | BootstrapperAppMSI;
export declare type OrganizationInfo = pulumiSetup.OrganizationInfo;
export interface BootstrapperAppSP {
  type: "sp";
  displayName: string;
  authentication: {
    rsaBits: number;
    tempDir: string;
    keyPath: string;
    certPath: string;
    certValidityPeriodDays: number;
    certSubject: string;
    pfxPath: string;
    pfxPassword: string;
  };
  willNeedToCreateAADApps: boolean;
  configSecretName: string;
}
export interface BootstrapperAppMSI {
  type: "msi";
  clientId: string;
  principalId: string;
}
export declare const performBootstrap: (inputs: Inputs) => Promise<{
  cicdRGName: string;
  kvName: string;
  pulumiConfigInfo: {
    backendConfig: pulumiAzure.PulumiAzureBackendConfig;
    auth: pulumiAzure.PulumiAzureBackendAuth;
  };
  envSpecificPipelineConfigReader: pulumi.EnvSpecificPipelineConfigReader;
}>;
export declare const constructVaultName: (organization: string) => string;
export declare const tryGetSecretValue: (
  auth: id.TokenCredential,
  secretReferenceOrURL: import("@data-heaving/azure-kv-secret").KevaultSecretReferenceOrURL,
) => Promise<string | undefined>;
export declare const BEGIN_CERTIFICATE: RegExp;
