"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = void 0;
const common = __importStar(require("@data-heaving/common"));
const pipeline = __importStar(require("@data-heaving/pulumi-azure-pipeline"));
const pulumiSetup = __importStar(require("@data-heaving/pulumi-azure-pipeline-setup"));
const bootstrap = __importStar(require("./bootstrap"));
const DEFAULT_STORAGE_CONTAINER_PREFIX = "cicd-env-";
const DEFAULT_KEY_NAME_PREFIX = "cicd-env-";
const DEFAULT_SECRET_NAME_PREFIX = "cicd-env-";
const main = async ({ eventEmitters, doChanges, credentials, bootstrapperApp, organization, azure, namingConventions, pipelineConfigs, }) => {
    const { bootstrapEventEmitter, pipelineEventEmitter, pulumiEventEmitters } = eventEmitters;
    const { cicdRGName, kvName, pulumiConfigInfo, envSpecificPipelineConfigReader, } = await bootstrap.performBootstrap({
        eventEmitter: bootstrapEventEmitter,
        credentials,
        azure,
        organization,
        pulumiEncryptionKeyBits: pipelineConfigs.pulumiEncryptionKeyBitsForBootstrapper,
        bootstrapperApp: getBootstrapAppForSetup(bootstrapperApp, organization),
    });
    pipelineEventEmitter?.emit("beforeRunningPulumiPortion", {
        organization: common.deepCopy(organization),
        doChanges,
    });
    // For some reason, due some TS compiler bug, we must specify generic argument explicitly.
    const command = doChanges ? "up" : "preview";
    return await pipeline.runPulumiPipeline({
        pulumi: {
            auth: pulumiConfigInfo.auth,
            backendConfig: pulumiConfigInfo.backendConfig,
            programArgs: {
                projectName: "bootstrap",
                stackName: "main",
                program: () => pulumiSetup.pulumiProgram({
                    organization,
                    envSpecificPipelineConfigReader,
                    pulumiPipelineConfig: {
                        auth: bootstrapperApp.type === "sp" &&
                            bootstrapperApp.envSpecificPulumiPipelineSPAuth
                            ? {
                                type: "sp",
                                certificateConfig: bootstrapperApp.envSpecificPulumiPipelineSPAuth,
                            }
                            : {
                                type: "msi",
                                sharedSARGName: cicdRGName,
                                sharedSAName: pulumiConfigInfo.backendConfig.storageAccountName,
                                containerPrefixString: namingConventions?.storageContainerPrefixString ??
                                    DEFAULT_STORAGE_CONTAINER_PREFIX,
                            },
                        pulumiKVInfo: {
                            rgName: cicdRGName,
                            name: kvName,
                            keyNamePrefix: namingConventions?.keyNamePrefix ?? DEFAULT_KEY_NAME_PREFIX,
                            secretNamePrefix: namingConventions?.secretNamePrefix ??
                                DEFAULT_SECRET_NAME_PREFIX,
                            encryptionKeyBits: pipelineConfigs.pulumiEncryptionKeyBitsForEnvSpecificPipeline,
                        },
                    },
                    targetResources: {
                        cicdRGSuffix: "bootstrap",
                        targetRGSuffix: undefined,
                        skipTargetRoleAssignment: true, // No "Owner" role assignment to subscription (as we already did it in bootstrap.default)
                    },
                }),
            },
        },
        azure: {
            tenantId: azure.tenantId,
            // Intentionally leave out subscription ID in order to cause errors if default provider is accidentally used.
            subscriptionId: undefined,
        },
    }, ["azure-native", "azuread", "tls"], command, pulumiEventEmitters);
};
exports.main = main;
const getBootstrapAppForSetup = (app, organization) => app.type === "msi"
    ? app
    : {
        ...app,
        willNeedToCreateAADApps: app.envSpecificPulumiPipelineSPAuth !== undefined &&
            organization.environments.length > 0,
    };
//# sourceMappingURL=pipeline.js.map