import * as common from "@data-heaving/common";
export const createBootstrapEventEmitterBuilder = () => new common.EventEmitterBuilder();
export const consoleLoggingBootstrapEventEmitterBuilder = (logSubscriptionId, logMessagePrefix, builder, consoleAbstraction) => {
    if (!builder) {
        builder = createBootstrapEventEmitterBuilder();
    }
    const logger = common.createConsoleLogger(logMessagePrefix, consoleAbstraction);
    const processStringWithSubscriptionID = (str) => {
        if (str && logSubscriptionId !== true) {
            // Remove first 51 characters ("/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx")
            str = str.substr(51);
            if (str.length === 0) {
                str = "whole subscription";
            }
        }
        return str;
    };
    const logRoleAssignment = (assignment, description) => logger(`Successfully assigned ${description}: ${assignment.principalId} (${assignment.principalType}) to role ${lastItem(assignment.roleDefinitionId?.split("/"))} on scope ${processStringWithSubscriptionID(assignment.scope)}`);
    builder.addEventListener("afterResolvingConfigReaderPrincipal", (arg) => logger(`Any stored SP authentication secrets will be readable by: ${JSON.stringify(arg)}`));
    builder.addEventListener("afterADApplicationExists", (arg) => logger(`Successfully ${arg.createNew ? "created" : "configured"} application with ID ${arg.application.id} and display name "${arg.application.displayName}".`));
    builder.addEventListener("afterADServicePrincipalExists", (arg) => logger(`Successfully ${arg.createNew ? "created" : "configured"} service principal with ID ${arg.servicePrincipal.id} connected to app ${arg.servicePrincipal.appId}.`));
    builder.addEventListener("beforeApplicationCredentialsExists", (arg) => logger(`For app ${arg.application.id}, will${arg.createNew ? "" : " not"} create credentials with ID ${arg.credential.keyId} and thumbprint ${arg.credential.customKeyIdentifier}.${arg.createNew
        ? `\nWaiting ${arg.waitTimeInSecondsIfCreated} seconds for credentials to sync.`
        : ""}`));
    builder.addEventListener("beforeApplicationHasEnoughPermissions", (arg) => logger(`For app ${arg.application.id}, will${arg.createNew ? "" : " not"} add permissons: ${JSON.stringify(arg.permissionsToAssign)}.${arg.createNew
        ? `\nWill wait ${arg.waitTimeInSecondsIfCreated} seconds for permissions to sync.`
        : ""}`));
    builder.addEventListener("beforeAdminConsentGranted", (arg) => logger(`For SP ${arg.servicePrincipal.id}, will${arg.createNew ? "" : " not"} grant admin consent for permissions: ${JSON.stringify(arg.permissionsToGrant)}.`));
    builder.addEventListener("bootstrapperRoleAssignmentCreatedOrUpdated", (arg) => logRoleAssignment(arg, "bootstrapper app resource privileges"));
    builder.addEventListener("resourceGroupCreatedOrUpdated", (arg) => logger(`Processed resource group "${arg.name}" to "${arg.location}".`));
    builder.addEventListener("storageAccountCreatedOrUpdated", (arg) => logger(`Processed storage account ${arg.name}.`));
    builder.addEventListener("storageAccountBlobServicesConfigured", (arg) => logger(`Configured storage account blob services: ${JSON.stringify(pick(arg, "isVersioningEnabled", "deleteRetentionPolicy", "cors", "containerDeleteRetentionPolicy", "restorePolicy", "sku"))}`));
    builder.addEventListener("storageAccountContainerCreatedOrUpdated", (arg) => logger(`Processed storage account container ${JSON.stringify(pick(arg, "name", "publicAccess"))}`));
    builder.addEventListener("keyVaultCreatedOrUpdated", (arg) => logger(`Processed key vault ${JSON.stringify(Object.assign(pick(arg, "name"), pick(arg.properties, "sku", "enablePurgeProtection", "enableRbacAuthorization", "enableSoftDelete", "enabledForDeployment", "enabledForDiskEncryption", "enabledForTemplateDeployment", "softDeleteRetentionInDays")))}`));
    builder.addEventListener("keyVaultAdminRoleAssignmentCreatedOrUpdated", (arg) => logRoleAssignment(arg, "bootstrapper app KV privileges"));
    builder.addEventListener("keyVaultAuthenticationSecretRoleAssignmentCreatedOrUpdated", (arg) => logRoleAssignment(arg, "bootstrapper app auth retriever privileges"));
    builder.addEventListener("keyCreatedOrUpdated", ({ createNew, key }) => logger(`Successfully ${createNew ? "created" : "retrieved"} key vault key "${key.name}" of type "${key.keyType}" having ${(key.key?.n?.byteLength ?? 0) * 8} bits.`));
    builder.addEventListener("authenticationSecretCreatedOrUpdated", ({ createNew, secretName }) => logger(`Successfully ${createNew ? "set" : "ensured existence of "} key vault secret "${secretName}"`));
    return builder;
};
const lastItem = (array) => array?.[array.length - 1];
// From https://stackoverflow.com/a/56592365
// TODO move these to @data-heaving/common
const pick = (obj, ...keys) => (obj
    ? Object.fromEntries(keys.filter((key) => key in obj).map((key) => [key, obj[key]]))
    : {});
// const inclusivePick = <T, TKey extends keyof T>(
//   obj: T,
//   ...keys: ReadonlyArray<TKey>
// ) => Object.fromEntries(keys.map((key) => [key, obj[key]])) as Pick<T, TKey>;
// const omit = <T, TKey extends keyof T>(obj: T, ...keys: ReadonlyArray<TKey>) =>
//   Object.fromEntries(
//     Object.entries(obj).filter(([key]) => !keys.includes(key as TKey)),
//   ) as Omit<T, TKey>;
//# sourceMappingURL=events.js.map