import * as t from "io-ts";
import * as validation from "@data-heaving/common-validation";
export const applicationRequiredResourceAccess = t.type({
    resourceAppId: validation.uuid,
    resourceAccess: t.array(t.type({
        type: validation.nonEmptyString,
        id: validation.uuid,
    }), "ResourceAccessList"),
}, "RequiredResourceAccess");
export const applicationKeyCredential = t.type({
    customKeyIdentifier: validation.nonEmptyString,
    displayName: t.string,
    endDateTime: validation.isoDateString,
    key: t.union([t.null, t.string]),
    keyId: validation.uuid,
    startDateTime: validation.isoDateString,
    // The following two properties are actually discriminating type union properties, but let's handle that later if needed
    type: t.union([
        t.literal("AsymmetricX509Cert"),
        t.literal("X509CertAndPassword"),
    ]),
    usage: t.union([t.literal("Verify"), t.literal("Sign")]),
}, "KeyCredential");
export const application = t.type({
    ["@odata.id"]: validation.urlWithPath,
    id: validation.uuid,
    appId: validation.uuid,
    displayName: t.string,
    keyCredentials: t.array(applicationKeyCredential),
    requiredResourceAccess: t.array(applicationRequiredResourceAccess, "RequiredResourceAccessList"),
    // And many others (e.g. passwordCredentials, but we don't use that (otherwise same as key credential, but without 'type' and 'usage', and 'hint' and 'secretText' instead ('secretText' and 'customKeyIdentifier' always null)))
}, "Application");
export const servicePrincipal = t.type({
    ["@odata.id"]: validation.urlWithPath,
    id: validation.uuid,
    appId: validation.uuid,
    displayName: t.string,
    appRoles: t.array(t.type({
        id: validation.uuid,
        origin: t.string,
        value: validation.nonEmptyString,
        description: t.union([t.string, t.null]),
        displayName: t.union([t.string, t.null]),
        // allowedMemberTypes: t.array(t.string) <- "Application" | ...?
    }, "AppRole"), "AppRoleList"),
    // And many others
    // They include also 'keyCredentials', however, that is not used for logging in at least, so we don't touch those here.
}, "ServicePrincipal");
export const servicePrincipalAppRoleAssignment = t.type({
    ["@odata.id"]: validation.urlWithPath,
    id: validation.nonEmptyString,
    appRoleId: validation.uuid,
    createdDateTime: validation.isoDateString,
    deletedDateTime: t.union([validation.isoDateString, t.null]),
    principalDisplayName: t.string,
    principalId: validation.uuid,
    principalType: t.string,
    resourceDisplayName: t.string,
    resourceId: validation.uuid,
});
export const AZURE_CLIENT_ID_ENV_NAME = "AZURE_CLIENT_ID";
//# sourceMappingURL=types.js.map