import * as pulumiAzure from "@data-heaving/pulumi-azure";
import * as pipeline from "@data-heaving/pulumi-azure-pipeline";
import * as pulumiSetup from "@data-heaving/pulumi-azure-pipeline-setup";
import * as bootstrap from "./bootstrap";
import * as types from "./types";
import * as events from "./events";
export interface Inputs {
  eventEmitters: PulumiPipelineEventEmitters;
  /**
   * If this is true, then Pulumi command `"up"` is executed. Otherwise, the Pulumi command `"preview"` is executed.
   */
  doChanges: boolean;
  credentials: bootstrap.BootstrappingCredentials;
  bootstrapperApp: BootstrapperApp;
  azure: pulumiAzure.AzureCloudInformationFull;
  organization: types.Organization;
  pipelineConfigs: {
    pulumiEncryptionKeyBitsForBootstrapper: number;
    pulumiEncryptionKeyBitsForEnvSpecificPipeline: number;
  };
  namingConventions?: NamingConventions;
}
export declare type PulumiPipelineEventEmitters = {
  bootstrapEventEmitter: bootstrap.BootstrapEventEmitter;
  pulumiEventEmitters: pipeline.PulumiPipelineEventEmitters;
  pipelineEventEmitter: events.PipelineBootstrapEventEmitter;
};
export interface NamingConventions {
  /**
   * This is used if:
   * - bootstrapperApp is MSI, or
   * - bootstrapperApp is SP, but env-specific Pulumi pipelines authentication is MSI (envSpecificPulumiPipelineSPAuth is set to undefined).
   *
   * Default value is `"cicd-env-"`.
   */
  storageContainerPrefixString?: string;
  /**
   * This is always needed. Default value is `"cicd-env-"`.
   */
  keyNamePrefix?: string;
  /**
   * This is always needed. Default value is `"cicd-env-"`.
   */
  secretNamePrefix?: string;
}
export declare type BootstrapperApp = BootstrapperAppSP | BootstrapperAppMSI;
export declare type BootstrapperAppSP = Omit<
  bootstrap.BootstrapperAppSP,
  "willNeedToCreateAADApps"
> & {
  /**
   * This exists only for SP info, as MSIs are unable to do any AAD changes.
   * Set this to non-undefined value in order to make env-specific Pulumi pipelines authenticate using SP.
   */
  envSpecificPulumiPipelineSPAuth?: pulumiSetup.SPCertificateInfo | undefined;
};
export declare type BootstrapperAppMSI = bootstrap.BootstrapperAppMSI;
export declare const main: ({
  eventEmitters,
  doChanges,
  credentials,
  bootstrapperApp,
  organization,
  azure,
  namingConventions,
  pipelineConfigs,
}: Inputs) => Promise<
  | import("@pulumi/pulumi/automation").UpResult
  | import("@pulumi/pulumi/automation").PreviewResult
>;
